package com.ruoyi.dhc.service.impl;

import com.ruoyi.common.core.domain.TreeSelect;
import com.ruoyi.common.core.domain.entity.DhcArea;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.dhc.mapper.DhcAreaMapper;
import com.ruoyi.dhc.service.IDhcAreaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 区域Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-07-29
 */
@Service
public class DhcAreaServiceImpl implements IDhcAreaService 
{
    @Autowired
    private DhcAreaMapper dhcAreaMapper;

    /**
     * 查询区域
     * 
     * @param areaId 区域主键
     * @return 区域
     */
    @Override
    public DhcArea selectDhcAreaByAreaId(Long areaId)
    {
        DhcArea area = dhcAreaMapper.selectDhcAreaByAreaId(areaId);
        if (area != null && area.getParentId() != null && area.getParentId() != 0L) {
            DhcArea parentArea = dhcAreaMapper.selectDhcAreaByAreaId(area.getParentId());
            area.setParentName(parentArea.getAreaName());
        }
        return area;
    }

    /**
     * 查询区域列表
     * 
     * @param dhcArea 区域
     * @return 区域
     */
    @Override
    public List<DhcArea> selectDhcAreaList(DhcArea dhcArea)
    {
        return dhcAreaMapper.selectDhcAreaList(dhcArea);
    }

    /**
     * 新增区域
     * 
     * @param dhcArea 区域
     * @return 结果
     */
    @Override
    public int insertDhcArea(DhcArea dhcArea)
    {
        dhcArea.setCreateTime(DateUtils.getNowDate());
        dhcArea.setDelFlag("0");
        Integer count = dhcAreaMapper.selectCount(dhcArea);
        if (count > 0) {
            return -1;
        } else {
            return dhcAreaMapper.insertDhcArea(dhcArea);
        }
    }

    /**
     * 修改区域
     * 
     * @param dhcArea 区域
     * @return 结果
     */
    @Override
    public int updateDhcArea(DhcArea dhcArea)
    {
        dhcArea.setUpdateTime(DateUtils.getNowDate());
        dhcArea.setDelFlag("0");
        Integer count = dhcAreaMapper.selectCount(dhcArea);
        if (count > 0) {
            return  -1;
        } else {
            return dhcAreaMapper.updateDhcArea(dhcArea);
        }
    }

    /**
     * 批量删除区域
     * 
     * @param areaIds 需要删除的区域主键
     * @return 结果
     */
    @Override
    public int deleteDhcAreaByAreaIds(Long[] areaIds)
    {
        return dhcAreaMapper.deleteDhcAreaByAreaIds(areaIds);
    }

    /**
     * 删除区域信息
     * 
     * @param areaId 区域主键
     * @return 结果
     */
    @Override
    public int deleteDhcAreaByAreaId(Long areaId)
    {
        DhcArea dhcArea = new DhcArea();
        dhcArea.setParentId(areaId);
        dhcArea.setDelFlag("0");
        Integer count = dhcAreaMapper.selectCount(dhcArea);
        if (count > 0) {
            return  -1;
        } else {
            return dhcAreaMapper.deleteDhcAreaByAreaId(areaId);
        }
    }

    /**
     * 查询区域树结构信息
     *
     * @param dhcArea 区域信息
     * @return 区域树信息集合
     */
    @Override
    public List<TreeSelect> selectAreaTreeList(DhcArea dhcArea)
    {
        List<DhcArea> areas = dhcAreaMapper.selectDhcAreaList(dhcArea);
        return buildAreaTreeSelect(areas);
    }

    /**
     * 构建前端所需要下拉树结构
     *
     * @param areas 区域列表
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildAreaTreeSelect(List<DhcArea> areas)
    {
        List<DhcArea> areaTrees = buildAreaTree(areas);
        return areaTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    /**
     * 构建前端所需要树结构
     *
     * @param areas 区域列表
     * @return 树结构列表
     */
    @Override
    public List<DhcArea> buildAreaTree(List<DhcArea> areas)
    {
        List<DhcArea> returnList = new ArrayList<DhcArea>();
        List<Long> tempList = areas.stream().map(DhcArea::getAreaId).collect(Collectors.toList());
        for (DhcArea area : areas)
        {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(area.getParentId()))
            {
                recursionFn(areas, area);
                returnList.add(area);
            }
        }
        if (returnList.isEmpty())
        {
            returnList = areas;
        }
        return returnList;
    }
    /**
     * 递归列表
     */
    private void recursionFn(List<DhcArea> list, DhcArea t)
    {
        // 得到子节点列表
        List<DhcArea> childList = getChildList(list, t);
        t.setChildren(childList);
        for (DhcArea tChild : childList)
        {
            if (hasChild(list, tChild))
            {
                recursionFn(list, tChild);
            }
        }
    }
    /**
     * 得到子节点列表
     */
    private List<DhcArea> getChildList(List<DhcArea> list, DhcArea t)
    {
        List<DhcArea> tlist = new ArrayList<DhcArea>();
        Iterator<DhcArea> it = list.iterator();
        while (it.hasNext())
        {
            DhcArea n = (DhcArea) it.next();
            if (StringUtils.isNotNull(n.getParentId()) && n.getParentId().longValue() == t.getAreaId().longValue())
            {
                tlist.add(n);
            }
        }
        return tlist;
    }
    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<DhcArea> list, DhcArea t)
    {
        return getChildList(list, t).size() > 0;
    }
}
